@@ -6,36 +6,35 @@
 Tools for analyzing beam and control point information from DICOM files
 Hierarchy of classes: Plan -> FxGroup -> Beam -> ControlPoint
 """
-# Copyright (c) 2016-2020 Dan Cutright
+# Copyright (c) 2016-2021 Dan Cutright
 # This file is part of DVH Analytics MLC Analyzer, released under a BSD license
 #    See the file LICENSE included with this distribution, also
 #    available at https://github.com/cutright/DVHA-MLCA
 
 import pydicom
-from pydicom.dataset import Dataset, DataElement
+from pydicom.dataset import Dataset
 import numpy as np
 from shapely.geometry import Polygon
 from shapely import speedups
 from mlca.utilities import (
     flatten_list_of_lists as flatten,
     get_xy_path_lengths,
+    run_multiprocessing,
 )
+from mlca.options import (
+    BEAM_MU_TOLERANCE,
+    CONTROL_POINT_MU_TOLERANCE,
+    CONTROL_POINT_POS_TOLERANCE,
+    DEFAULT_OPTIONS,
+)
+import warnings
 
-BEAM_MU_TOLERANCE = 0.001
-CONTROL_POINT_MU_TOLERANCE = 0.00001
-CONTROL_POINT_POS_TOLERANCE = 0.0001
-
-DEFAULT_OPTIONS = {
-    "max_field_size_x": 400.0,
-    "max_field_size_y": 400.0,
-    "complexity_weight_x": 1.0,
-    "complexity_weight_y": 1.0,
-}
 
 COLUMNS = [
     "Patient Name",
     "Patient MRN",
     "Study Instance UID",
+    "SOP Instance UID",
     "TPS",
     "Plan name",
     "# of Fx Group(s)",
@@ -84,31 +83,73 @@ class PlanSet:
     ----------
     file_paths : list
         A list of file paths to DICOM-RT Plan files
+    verbose : bool, optional
+        Set to true to print detailed information (ignored if
+        multiprocessing enabled)
+    processes : int
+        Number of parallel processes allowed
 
     """
 
-    def __init__(self, file_paths, verbose=False, **kwargs):
+    def __init__(self, file_paths, verbose=False, processes=1, **kwargs):
+        self.file_paths = file_paths
+        self.verbose = verbose
+        self.processes = processes
+        self.kwargs = kwargs
+        self.summary_table = [",".join(COLUMNS)]
 
-        self.plans = []
-        plan_count = len(file_paths)
-        for i, file_path in enumerate(file_paths):
-            if verbose:
-                print(
-                    "Analyzing (%s of %s): %s" % (i + 1, plan_count, file_path)
-                )
-            self.plans.append(Plan(file_path, **kwargs))
-            if verbose:
-                print(self.plans[-1], "\n")
+        if processes == 1:
+            try:
+                self._run()
+            except KeyboardInterrupt:
+                print("Plan analyzer halted!")
+        else:
+            data = run_multiprocessing(
+                self._worker, self.file_paths, self.processes
+            )
+            for row in data:
+                self.summary_table.extend(row)
 
-    @property
-    def summary_table(self):
-        """Get a summary """
-        summary_table = [",".join(COLUMNS)]
-        for plan in self.plans:
+    def _run(self):
+        """Process files, accumulate data in self.summary_table"""
+        plan_count = len(self.file_paths)
+        for i, file_path in enumerate(self.file_paths):
+            print("Analyzing (%s of %s): %s" % (i + 1, plan_count, file_path))
+            try:
+                plan = Plan(file_path, **self.kwargs)
+                for fx_grp_row in plan.summary:
+                    row = [fx_grp_row[key] for key in COLUMNS]
+                    self.summary_table.append(",".join(row))
+
+                if self.verbose:
+                    print(plan, "\n")
+            except Exception as e:
+                print("Analysis failed\n%s\n" % e)
+
+    def _worker(self, file_path):
+        """Multiprocessing worker
+
+        Parameters
+        ----------
+        file_path : str
+            file path of a DICOM-RT Plan file
+
+        Returns
+        -------
+        list
+            Results from Plan.summary prepped for CSV output
+        """
+        data = []
+        try:
+            with warnings.catch_warnings():
+                warnings.simplefilter("ignore")
+                plan = Plan(file_path, **self.kwargs)
             for fx_grp_row in plan.summary:
                 row = [fx_grp_row[key] for key in COLUMNS]
-                summary_table.append(",".join(row))
-        return summary_table
+                data.append(",".join(row))
+        except Exception:
+            pass
+        return data
 
     @property
     def csv(self):
@@ -153,6 +194,7 @@ class Plan:
                 "Patient Name": self.patient_name,
                 "Patient MRN": self.patient_id,
                 "Study Instance UID": self.study_instance_uid,
+                "SOP Instance UID": self.sop_instance_uid,
                 "TPS": self.tps,
                 "Plan name": self.plan_name,
                 "# of Fx Group(s)": str(len(self.fx_group)),
@@ -173,6 +215,7 @@ class Plan:
             "Patient Name:        %s" % self.patient_name,
             "Patient MRN:         %s" % self.patient_id,
             "Study Instance UID:  %s" % self.study_instance_uid,
+            "SOP Instance UID:    %s" % self.sop_instance_uid,
             "TPS:                 %s" % self.tps,
             "Plan name:           %s" % self.plan_name,
             "# of Fx Group(s):    %s" % len(self.fx_group),
@@ -213,6 +256,7 @@ class Plan:
             "patient_name",
             "patient_id",
             "study_instance_uid",
+            "sop_instance_uid",
             "tps",
         ]
         if not all([getattr(self, a) == getattr(other, a) for a in attr]):
@@ -232,7 +276,7 @@ class Plan:
         str
             RTPlanLabel (300A,0002)
         """
-        return str(self.rt_plan.RTPlanLabel)
+        return str(getattr(self.rt_plan, "RTPlanLabel", ""))
 
     @property
     def patient_name(self):
@@ -243,7 +287,7 @@ class Plan:
         str
             PatientName (0010,0010)
         """
-        return str(self.rt_plan.PatientName)
+        return str(getattr(self.rt_plan, "PatientName", ""))
 
     @property
     def patient_id(self):
@@ -254,7 +298,7 @@ class Plan:
         str
             PatientID (0010,0020)
         """
-        return str(self.rt_plan.PatientID)
+        return str(getattr(self.rt_plan, "PatientID", ""))
 
     @property
     def study_instance_uid(self):
@@ -265,7 +309,18 @@ class Plan:
         str
             StudyInstanceUID (0020,000D)
         """
-        return str(self.rt_plan.StudyInstanceUID)
+        return str(getattr(self.rt_plan, "StudyInstanceUID", ""))
+
+    @property
+    def sop_instance_uid(self):
+        """Get the patient study instance UID
+
+        Returns
+        -------
+        str
+            SOPInstanceUID (0008,0018)
+        """
+        return str(getattr(self.rt_plan, "SOPInstanceUID", ""))
 
     @property
     def tps(self):
@@ -277,8 +332,8 @@ class Plan:
             Manufacturer (0008,0070) and ManufacturerModelName (0008,1090)
         """
         return "%s %s" % (
-            self.rt_plan.Manufacturer,
-            self.rt_plan.ManufacturerModelName,
+            getattr(self.rt_plan, "Manufacturer", ""),
+            getattr(self.rt_plan, "ManufacturerModelName", ""),
         )
 
     @property
@@ -479,7 +534,7 @@ class Beam:
         ]
 
         self.summary = {
-            "cp": range(1, len(self.control_point) + 1),
+            "cp": list(range(1, len(self.control_point) + 1)),
             "cum_mu_frac": [cp.cum_mu for cp in self.control_point],
             "cum_mu": self.cum_mu,
             "cp_mu": self.cp_mu,
@@ -760,16 +815,18 @@ class Beam:
             self.options["complexity_weight_x"],
             self.options["complexity_weight_y"],
         )
-        return (
-            np.divide(
-                np.multiply(
-                    np.add(c1 * self.perimeter_x, c2 * self.perimeter_y),
-                    self.cp_mu,
-                ),
-                self.area,
+        if self.meter_set and self.meter_set > 0:
+            return (
+                np.divide(
+                    np.multiply(
+                        np.add(c1 * self.perimeter_x, c2 * self.perimeter_y),
+                        self.cp_mu,
+                    ),
+                    self.area,
+                )
+                / self.meter_set
             )
-            / self.meter_set
-        )
+        return 0
 
 
 class ControlPoint:
